program RegLazCL;

{$H+}

uses
  SysUtils,
  {$IFDEF MSWINDOWS}
  Registry, shlobj,
  {$ENDIF}
  PPDK_RegFileType in '..\..\..\UNITS\PPDK_RegFileType.pas';

{$R *.res}

ResourceString
  cID_ERROR = 'Error: ';
  cID_HELP  = 'Help: ';
  cID_WARN  = 'Warning: ';
  cID_INFO  = 'Information: ';

const
  cRET_OK                    = 0;
  cRET_OK_EXT_NOT_REGISTERED = 1;
  cRET_ERR_GENERIC           = 2;
  cRET_ERR_VALUE_EXPECTED    = 3;
  cRET_ERR_NO_VALUE_EXPECTED = 4;
  cRET_ERR_INVALID_PARAM     = 5;
procedure ShowHelp();
var
  sFileName: String;
begin
  sFileName := ChangeFileExt(ExtractFileName(ParamStr(0)), '');
	WriteLn('');
	WriteLn('');
	WriteLn(sFileName + ' - Register filetypes with Lazarus');
	WriteLn('');
	WriteLn('Usage: ' + sFileName + ' -c<[.]ext> -s<[.]ext> -u<[.]ext> -h');
	WriteLn('');
	WriteLn('Parameters:');
	WriteLn('-c  Checks the given file extention if is registered with the Lazarus IDE.');
	WriteLn('    Takes one value, the extention to be checked, with or without the dot.');
	WriteLn('    e.g: ' + sFileName + ' -clpi');
	WriteLn('         ' + sFileName + ' -c.lpi');
	WriteLn('         ' + sFileName + ' -clpi -clpr');
	WriteLn('');
	WriteLn('-s  Registers the given file extention to open with the Lazarus IDE.');
	WriteLn('    Takes one value, the extention to be registered, with or without the dot.');
	WriteLn('    e.g: ' + sFileName + ' -slpi');
	WriteLn('         ' + sFileName + ' -s.lpi');
	WriteLn('         ' + sFileName + ' -slpi -slpr');
	WriteLn('');
	WriteLn('-u  Removes the assocciation between given file extention and Lazarus IDE.');
	WriteLn('    Takes one value, the extention to be unregistered, with or without the dot.');
	WriteLn('    e.g: ' + sFileName + ' -ulpi');
	WriteLn('         ' + sFileName + ' -u.lpi');
	WriteLn('         ' + sFileName + ' -ulpi -ulpr');
	WriteLn('');
	WriteLn('-p  Program waits for user input before exiting.');
	WriteLn('    Takes no value.');
	WriteLn('    e.g: ' + sFileName + ' -p');
	WriteLn('');
	WriteLn('-h  Shows this help screen.');
	WriteLn('    Takes no value.');
	WriteLn('    e.g: ' + sFileName + ' -h -p');
	WriteLn('         ' + sFileName + ' -h');
	WriteLn('');
end;

procedure ShowMsg(sCaption, sText: String);
begin
  if(sCaption[1] = 'H')then
  begin
    ShowHelp;
  end
  else
  begin
  	WriteLn(sCaption + sText);
  end;
end;

var
  sOpt,
  sVal,
  srDESC,
  srLDESC,
  srMNUTXT,
  srCMD,
  srICON: String;
  nidx,
  nRdln,
  nlen,
  nRetCode: Integer;
  Reg: TRegistry;
begin
  nRdln := 0;
  nRetCode := cRET_OK;
  if(ParamCount > 0)then
  begin
    Reg := TRegistry.Create();
  	for nidx := 1 to ParamCount do
  	begin
      nlen := Length(ParamStr(nidx));
	    sOpt := LeftStr(ParamStr(nidx), 2);
	    sVal := RightStr(ParamStr(nidx), nlen - 2);
      sOpt := LowerCase(sOpt);
      if(nlen < 2)then
      begin
        ShowMsg(cID_ERROR, 'Invalid parameter: ' + sOpt);
        nRetCode := cRET_ERR_INVALID_PARAM;
      end
      else
      if(sVal = '')then
      begin
        if(sOpt = '-h')then
        begin
          ShowMsg(cID_HELP, '');
        end
        else
        if(sOpt = '-p')then
        begin
          nRdln := 1;
        end
        else
        begin
          ShowMsg(cID_WARN, 'Parameter #' + IntToStr(nidx) + ' (' + sOpt + ') has no value.');
          nRetCode := cRET_ERR_VALUE_EXPECTED;
        end;
      end
      else
      begin
        if(sOpt[2] in ['c','s','u'])then
        begin
          case sOpt[2] of
          	'c': { Check a file assocciation }
            begin
              if(GetFileExt(Reg, sVal))then
              begin
                ShowMsg(cID_INFO, 'Extension ''' + sVal + ''' is REGISTERED with Lazarus IDE.');
              end
              else
              begin
                ShowMsg(cID_WARN, 'Extension ''' + sVal + ''' is NOT registered with Lazarus IDE.');
                nRetCode := cRET_OK_EXT_NOT_REGISTERED;
              end;
            end;
            's': { Set a file assocciation }
            begin
            	srMNUTXT := 'Open with Lazarus IDE';
              srCMD    := 'c:\lazarus\lazarus.exe "%1"';
              if(Pos('lpi', sVal) > 0)then
              begin
              	srDESC   := 'LazarusProject';
              	srLDESC  := 'Lazarus Project Information';
              	srICON   := 'c:\lazarus\images\LazarusProject.ico';
              end
              else
              if(Pos('lpr', sVal) > 0)then
              begin
              	srDESC   := 'LazarusProjectSource';
              	srLDESC  := 'Lazarus Project Main Source';
              	srICON   := 'c:\lazarus\images\lprfile.ico';
              end
              else
              if(Pos('lpk', sVal) > 0)then
              begin
              	srDESC   := 'LazarusPackage';
              	srLDESC  := 'Lazarus Package File';
              	srICON   := 'c:\lazarus\images\lazaruspackage.ico';
              end
              else
              if(Pos('lfm', sVal) > 0)then
              begin
              	srDESC   := 'LazarusForm';
              	srLDESC  := 'Lazarus Form File';
              	srICON   := 'c:\lazarus\images\LazarusForm.ico';
              end
              else
              if(Pos('pas', sVal) > 0)then
              begin
              	srDESC   := 'LazarusUnit';
              	srLDESC  := 'Object Pascal Unit';
              	srICON   := 'c:\lazarus\images\LazarusSource.ico';
              end
              else
              if(Pos('pp', sVal) > 0)then
              begin
              	srDESC   := 'LazarusFPCUnit';
              	srLDESC  := 'Object Pascal Unit';
              	srICON   := 'c:\lazarus\images\LazarusSource.ico';
              end
              else
              if(Pos('inc', sVal) > 0)then
              begin
              	srDESC   := 'LazarusInclude';
              	srLDESC  := 'Object Pascal Include File';
              	srICON   := 'c:\lazarus\images\includefile.ico';
              end;
              if(SetFileExt(Reg, sVal, srDESC, srLDESC, srMNUTXT, srCMD, srICON))then
              begin
                SHChangeNotify(SHCNE_ASSOCCHANGED, SHCNF_IDLIST, nil, nil);
                ShowMsg(cID_INFO, 'Assocciation of ''' + sVal + ''' file type is succesful.');
              end
              else
              begin
                ShowMsg(cID_ERROR, 'Assocciation of ''' + sVal + ''' file type failed.');
                nRetCode := cRET_ERR_GENERIC;
              end;
            end;
            'u': { UnSet file accossiation }
            begin
              if((Pos('lpi', sVal) > 0) or
              	 (Pos('lpr', sVal) > 0) or
                 (Pos('lpk', sVal) > 0) or
                 (Pos('lfm', sVal) > 0) or
                 (Pos('pas', sVal) > 0) or
                 (Pos('pp', sVal) > 0)  or
                 (Pos('inc', sVal) > 0))then
              begin
                if(GetFileExt(Reg, sVal))then
                begin
              		if(UnSetFileExt(Reg, sVal))then
                	begin
	                  SHChangeNotify(SHCNE_ASSOCCHANGED, SHCNF_IDLIST, nil, nil);
	                  ShowMsg(cID_INFO, 'Disassocciation of ''' + sVal + ''' file type is succesful.');
                    nRetCode := cRET_OK;
	                end
	                else
	                begin
	                	ShowMsg(cID_ERROR, 'Disassocciation of ''' + sVal + ''' file type failed.');
                    nRetCode := cRET_ERR_GENERIC;
	                end;
                end
                else
                begin
									ShowMsg(cID_WARN, 'Extension ''' + sVal + ''' is NOT registered with Lazarus IDE.' + #13 + 'Nothing to delete...');
                  nRetCode := cRET_OK_EXT_NOT_REGISTERED;
                end;
              end;
            end;
          end;
        end
        else
        begin
          if(sOpt[2] = 'h')then
          begin
            ShowMsg(cID_INFO, 'Parameter ''-h'' takes NO value!');
            ShowMsg(cID_HELP, '');
            nRetCode := cRET_ERR_NO_VALUE_EXPECTED;
          end
          else
          begin
						ShowMsg(cID_ERROR, 'Invalid parameter: ' + sOpt);
            nRetCode := cRET_ERR_INVALID_PARAM;
          end;
        end;
      end;
    end;
    FreeAndNil(Reg);
  end
  else
  begin
    ShowMsg(cID_HELP, '');
  end;
  if(nRdln > 0)then
  begin
  	ReadLn;
  end;
  ExitCode := nRetCode;
end.

